# All directories mentioned here are relative to the project root
ifndef BASEDIR
  BASEDIR:=.
endif

ifndef TARGET         # Default target is to the body
  TARGET:=body
endif

SRCBASEDIR:=$(BASEDIR)/src

INCLUDEDIR:=$(BASEDIR)/src/include
OBJDIR:=$(BASEDIR)/targets/$(TARGET)/build
BINDIR:=$(BASEDIR)/targets/$(TARGET)/bin
LIBDIR:=$(BASEDIR)/targets/$(TARGET)/lib
TMPDIR:=/tmp

# Common flags
# Ackley: -Wextra is just too much, I think..
#COMMON_CFLAGS:=-Wall -pedantic -Werror -Wundef -Winline -Wextra     # Let's help them help us
#COMMON_CPPFLAGS:=-Wall -pedantic -Werror -Wundef -Winline -Wextra   #   "
COMMON_CFLAGS:=-Wall -pedantic -Werror -Wundef     # Let's help them help us
COMMON_CPPFLAGS:=-Wall -pedantic -Werror -Wundef   # Let's help them help us
COMMON_LDFLAGS:=--fatal-warnings --gc-sections -L$(LIBDIR)  #   "  (and save space)

CORE_EXTRA_CFLAGS:=-Winline --param max-inline-insns-single=24
CORE_EXTRA_CPPFLAGS:=-Winline --param max-inline-insns-single=24
CORE_EXTRA_LDFLAGS:=

SKETCH_EXTRA_CFLAGS:=
SKETCH_EXTRA_CPPFLAGS:=
SKETCH_EXTRA_LDFLAGS:=

# For NATIVE_ARCH_CODE and CROSS_ARCH_CODE, below:
# Possible *_ARCH_CODE values: 
#   1==SFBoard Rev A 
#   2==SFBoard Rev B/C
#  20==Linux
#( 30==Windows?)
#( 40==MacOS?)
#
#  ARCH_CODE must be specific enough to ensure full binary code compatibility

# Native tool chain
NATIVE_OS:=POSIX
#Or: NATIVE_OS=WIN32
NATIVE_ARCH_CODE:=20
NATIVE_GCC:=gcc
NATIVE_GPP:=g++
NATIVE_GCC_CFLAGS:=$(COMMON_CFLAGS)
NATIVE_GCC_CPPFLAGS:=$(COMMON_CPPFLAGS)
NATIVE_GCC_LDFLAGS:=$(COMMON_LDFLAGS)
NATIVE_GCC_DEFINES:=-DHOST_MODE -D HOST_$(NATIVE_OS)
NATIVE_GCC_OPTS:=-O0
NATIVE_GCC_THUMB:= 
NATIVE_GCC_DEBUGS:=-g3
NATIVE_LD:=ld
NATIVE_OBJCOPY:=objcopy
NATIVE_DOXYGEN:=/usr/local/bin/doxygen

# Cross tool chain
CROSS_OS:=error-no-os
CROSS_ARCH_CODE:=2
CROSS_BASE:=/home/ackley/ARM/arm-2008q3
CROSS_GCC:=$(CROSS_BASE)/bin/arm-none-eabi-gcc
CROSS_GPP:=$(CROSS_BASE)/bin/arm-none-eabi-g++
CROSS_GCC_CFLAGS:=$(COMMON_CFLAGS) -mcpu=arm7tdmi -mthumb-interwork -mno-long-calls -ffunction-sections -fdata-sections
CROSS_GCC_CPPFLAGS:=$(COMMON_CPPFLAGS) -mcpu=arm7tdmi -mthumb-interwork -mno-long-calls -ffunction-sections -fdata-sections -fno-rtti -fno-exceptions -fno-use-cxa-atexit -fno-use-cxa-get-exception-ptr
CROSS_GCC_LDFLAGS:=$(COMMON_LDFLAGS) -L $(CROSS_BASE)/lib/gcc/arm-none-eabi/4.3.2
CROSS_GCC_DEFINES:=
CROSS_GCC_OPTS:= -O2 -Os
CROSS_GCC_THUMB:= 
CROSS_GCC_DEBUGS:=
CROSS_LD:=$(CROSS_BASE)/bin/arm-none-eabi-ld
CROSS_OBJCOPY:=$(CROSS_BASE)/bin/arm-none-eabi-objcopy
CROSS_DOXYGEN:=error-no-doxygen

ifeq ($(TARGET),thumb)
 CROSS_GCC_THUMB+=-mthumb	
 # Allow more instructions before warning in thumb mode
 CORE_EXTRA_CFLAGS:=-Winline --param max-inline-insns-single=48
 CORE_EXTRA_CPPFLAGS:=-Winline --param max-inline-insns-single=48

endif

